#!/bin/sh

PATH=/sbin:/usr/sbin:/bin:/usr/bin

UDHCPD=/usr/sbin/udhcpd
PATH_UDHCPD_CONF=/etc/udhcpd.conf
PATH_UDHCPD_PID=/var/run/udhcpd.pid
PATH_UDHCPD_LEASES_DIR=/var/lib/misc/
UDHCPD_LEASES_FILE=udhcpd.leases
MAX_LEASES=65536
DEFAULT_LEASE=864000

#
IFNAME=$1
COMMAND=$2

# Fetch configuration file
if [ -e /etc/init.d/dhcpscfg ] ; then
   . /etc/init.d/dhcpscfg
fi

# -------------------------------------------------------------------
DHCPSSADDR=${DHCPSSADDR:-0.0.0.0}
DHCPSEADDR=${DHCPSEADDR:-0.0.0.0}
DHCPSMASK=${DHCPSMASK:-0.0.0.0}
DHCPSROUTER=${DHCPSROUTER:-0.0.0.0}
DHCPSLEASE=${DHCPSLEASE:-864000}

# -------------------------------------------------------------------
# Create DHCPD Configuration File
# -------------------------------------------------------------------
udhcpd_create_conf() {
#create dhcp conf-file
	echo "Creating UDHCPD Conf-file" > /dev/console

	# Start Address Check
	if [ ! -n ${DHCPSSADDR} -o ${DHCPSSADDR} = "0.0.0.0" ]; then
		return 1
	fi

	# End Address Check
	if [ ! -n ${DHCPSEADDR} -o ${DHCPSEADDR} = "0.0.0.0" ]; then
		return 1
	fi

	# Address Range Check
	I=1
	while [ $I -le 4 ]; do
		SA=`echo ${DHCPSSADDR} | cut -f ${I} -d'.'`
		EA=`echo ${DHCPSEADDR} | cut -f ${I} -d'.'`
		if [ ${EA} -gt ${SA} ]; then
			break
		elif [ ${SA} -gt ${EA} ]; then
			return 1
		fi
		I=`expr $I + 1`
	done

	# Subnet Mask Check
	_mask="255.255.255.0"

	if [ -n ${DHCPSMASK} -a ${DHCPSMASK} != "0.0.0.0" ]; then
		_mask="${DHCPSMASK}"
	else 
		_mask=`ipcalc -m ${DHCPSSADDR} | cut -f2 -d'='`
	fi

	# Network Address Check
	_netaddrs=`ipcalc -n ${DHCPSSADDR} ${_mask} | cut -f2 -d'='`
	_netaddre=`ipcalc -n ${DHCPSEADDR} ${_mask} | cut -f2 -d'='`
	if [ ${_netaddrs} != ${_netaddre} ]; then
		return 1
	fi

	# Broad cast address
	_bcast=`ipcalc -b ${DHCPSSADDR} ${_mask} | cut -f2 -d'='`

	# Create File
cat >${PATH_UDHCPD_CONF}<<__EOF__
	start ${DHCPSSADDR}
	end   ${DHCPSEADDR}
__EOF__

	# Write Interface
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	interface ${IFNAME}
__EOF__

	# Write MaxLeases
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	max_leases ${MAX_LEASES}
__EOF__

	# Write LeaseFilePath
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	lease_file ${PATH_UDHCPD_LEASES_DIR}${UDHCPD_LEASES_FILE}
__EOF__

	# Write PidFilePath
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	pidfile ${PATH_UDHCPD_PID}
__EOF__

	# Write SubnetMask
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	option subnet ${_mask}
__EOF__

	# Write Gateway
	if [ -n ${DHCPSROUTER} -a ${DHCPSROUTER} != "0.0.0.0" ]; then
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	option router ${DHCPSROUTER}
__EOF__
	fi

	# Write Broadcast
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	option broadcast ${_bcast}
__EOF__

	if [ -n ${DHCPSLEASE} ]; then
		# lease time = 0 is default lease time
		if [ ${DHCPSLEASE} = "0" ]; then
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	option lease ${DEFAULT_LEASE}
__EOF__
		else
			_time=`expr ${DHCPSLEASE} \* 60`
cat >>${PATH_UDHCPD_CONF}<<__EOF__
	option lease ${_time}
__EOF__
		fi
	fi

	echo "--------------" > /dev/console
	cat ${PATH_UDHCPD_CONF} > /dev/console
	echo "--------------" > /dev/console

	return 0
}

udhcpd_start() {
	if [ -n "`ps | grep ${UDHCPD} | grep ${PATH_UDHCPD_CONF}`" ]; then
		echo "udhcpd already run"
		return
	fi

	if ! grep `echo ${IFNAME}: | cut -f 1 -d :`: /proc/net/dev 1> /dev/null ; then
		echo "udhcpd I/F:${IFNAME} is not available!!" > /dev/console
		return
	fi

	udhcpd_create_conf
	if [ ${?} != "0" ]; then
		echo "udhcpd cannot create configuration File!!" > /dev/console
		return
	fi

	if [ ! -d ${PATH_UDHCPD_LEASES_DIR} ]; then
		mkdir -m 755 -p ${PATH_UDHCPD_LEASES_DIR}
	fi

	if [ -x ${DHCPD} ]; then
		echo "Starting DHCP Server daemon ${IFNAME}:" > /dev/console
		/sbin/route add -host 255.255.255.255 dev ${IFNAME}
		${UDHCPD} ${PATH_UDHCPD_CONF}
	fi
}

udhcpd_sysStart() {
	if [ $USEDHCPS -ne 0 ] ; then
		udhcpd_start
	else
	    echo "DHCP Server setting is disabled ${IFNAME}:" > /dev/console
	fi
}

udhcpd_stop() {
	if [ -f  ${PATH_UDHCPD_PID} ]; then
		echo "Stopping DHCP Server daemon" > /dev/console
		/sbin/route del -host 255.255.255.255 dev ${IFNAME}
		kill `cat ${PATH_UDHCPD_PID}`
	fi
}

udhcpd_restart() {
  udhcpd_stop
  sleep 2
  udhcpd_start
}

# ===================================================================
# Main #
# -------------------------------------------------------------------
if [ "${COMMAND}" != "" -a "${IFNAME}" != "" ]; then
	case "${COMMAND}" in
	'sysstart')
	  udhcpd_sysStart
	  ;;
	'start')
	  udhcpd_start
	  ;;
	'stop')
	  udhcpd_stop
	  ;;
	'restart')
	  udhcpd_restart
	  ;;
	*)
	esac
else
	echo "usage $0 IFNAME sysstart|start|stop|restart" > /dev/console
fi

exit 0
